"(" @punctuation.bracket
")" @punctuation.bracket
"[&&NHX" @punctuation.bracket
"]" @punctuation.bracket

(name) @function
(length) @number
(nhx_entry key: (nhx_val) @keyword value: (nhx_val) @string.special)
